#include "datatypes.h"


//////// Fielding ////////

Uint32 getLong(BYTE * Message, const Uint32 Offset)
{
	return *(Uint32*)&(Message[Offset]);
}

Uint16 getShort(BYTE * Message, const Uint32 Offset)
{
	return *(Uint16*)&(Message[Offset]);
}

Uint32 getLong(char * Message, const Uint32 Offset)
{
	return *(Uint32*)&(Message[Offset]);
}

Uint16 getShort(char * Message, const Uint32 Offset)
{
	return *(Uint16*)&(Message[Offset]);
}

BYTE getByte(char * Message, const Uint32 Offset)
{
	return *(BYTE*)&(Message[Offset]);
}


//////// Linked list node ////////

TEMPLATE _listnode <AnonymousStruct>::_listnode (AnonymousStruct * nItem)
{
	item = nItem;

	next = NULL;
	last = NULL;
}

TEMPLATE _listnode <AnonymousStruct>::~_listnode()
{
	if (item) delete item;
}


//////// Linked list ////////

TEMPLATE _linkedlist <AnonymousStruct>::_linkedlist()
{
	total	= 0;

	head	= NULL;
	tail	= NULL;
}

TEMPLATE _linkedlist <AnonymousStruct>::~_linkedlist()
{
	clear(false);
}

TEMPLATE void _linkedlist <AnonymousStruct>::clear(bool nullify)
{
	if (head)
	{
		_listnode <AnonymousStruct> * parse = head;

		total = 0;

		if (nullify)
		{
			_listnode <AnonymousStruct> * next;

			while (parse)
			{
				next = parse;

				parse = parse->next;

				next->item = NULL;

				delete next;
			}
		}
		else
		{
			_listnode <AnonymousStruct> * next;

			while (parse)
			{
				next = parse;

				parse = parse->next;

				delete next;
			}
		}

		head = NULL;
	}

	tail = NULL;
}

TEMPLATE void _linkedlist <AnonymousStruct>::prepend(AnonymousStruct * item)
{
	_listnode <AnonymousStruct> * new_head = new _listnode <AnonymousStruct> (item);

	new_head->last = NULL;
	new_head->next = head;

	if (tail == NULL)
		tail = new_head;
	else
		head->last = new_head;

	head = new_head;

	total++;
}

TEMPLATE void _linkedlist <AnonymousStruct>::append(AnonymousStruct * item)
{
	_listnode <AnonymousStruct> * new_tail = new _listnode <AnonymousStruct> (item);

	new_tail->last = tail;
	new_tail->next = NULL;

	if (head == NULL)
		head = new_tail;
	else
		tail->next = new_tail;

	tail = new_tail;

	total++;
}

TEMPLATE void _linkedlist <AnonymousStruct>::insertAfter(_listnode <AnonymousStruct> * node, AnonymousStruct * item)
{
	if (node == tail)
	{
		append(item);
	}
	else
	{
		_listnode <AnonymousStruct> * lastnode = node;
		_listnode <AnonymousStruct> * nextnode = node->next;
		_listnode <AnonymousStruct> * new_node = new _listnode <AnonymousStruct> (item);

		new_node->last = lastnode;
		new_node->next = nextnode;
		if (lastnode) lastnode->next = new_node;
		if (nextnode) nextnode->last = new_node;

		total++;
	}
}

TEMPLATE void _linkedlist <AnonymousStruct>::insertBefore(_listnode <AnonymousStruct> * node, AnonymousStruct * item)
{
	if (node == head)
	{
		prepend(item);
	}
	else
	{
		_listnode <AnonymousStruct> * lastnode = node->last;
		_listnode <AnonymousStruct> * nextnode = node;
		_listnode <AnonymousStruct> * new_node = new _listnode <AnonymousStruct> (item);

		new_node->last = lastnode;
		new_node->next = nextnode;
		if (lastnode) lastnode->next = new_node;
		if (nextnode) nextnode->last = new_node;

		total++;
	}
}

TEMPLATE void _linkedlist <AnonymousStruct>::kill(_listnode <AnonymousStruct> * node, bool nullify)
{
	if (tail == node)
	{
		tail = tail->last;
		if (tail) tail->next = NULL;
	}
	else
	{
		if (node->last) node->last->next = node->next;
	}

	if (head == node)
	{
		head = head->next;
		if (head) head->last = NULL;
	}
	else
	{
		if (node->next) node->next->last = node->last;
	}

	if (nullify)
		node->item = NULL;

	delete node;

	total--;
}

TEMPLATE void _linkedlist <AnonymousStruct>::kill(AnonymousStruct * item, bool nullify)
{
	_listnode <AnonymousStruct> * parse = head;

	while (parse)
	{
		if (parse->item == item)
		{
			kill(parse, nullify);
			return;
		}

		parse = parse->next;
	}
}


//////// Jump table ////////

TEMPLATE _jumptable <AnonymousStruct>::_jumptable()
{
	clear();
}

TEMPLATE void _jumptable <AnonymousStruct>::clear()
{
	memset(RPC, 0, 1024);	// 256 * sizeof(void*)
}

TEMPLATE void _jumptable <AnonymousStruct>::add(BYTE index, void (*function)(AnonymousStruct *))
{
	RPC[index] = function;
}

TEMPLATE void _jumptable <AnonymousStruct>::kill(BYTE index)
{
	RPC[index] = NULL;
}

TEMPLATE bool _jumptable <AnonymousStruct>::call(BYTE index, AnonymousStruct * params)
{
	if (RPC[index])
	{
		RPC[index](params);
		return true;
	}

	return false;
}


//////// Reference table ////////

TEMPLATE inline _referencetable <AnonymousStruct>::_referencetable()
{
	clear();
}

TEMPLATE void _referencetable <AnonymousStruct>::clear()
{
	memset(LIST, 0, 1024);	// 256 * sizeof(void*)
}

TEMPLATE void _referencetable <AnonymousStruct>::add(BYTE index, AnonymousStruct * data)
{
	LIST[index] = data;
}

TEMPLATE void _referencetable <AnonymousStruct>::kill(BYTE index)
{
	LIST[index] = NULL;
}

TEMPLATE AnonymousStruct * _referencetable <AnonymousStruct>::get(BYTE index)
{
	return LIST[index];
}
